#############################################################################
#############################################################################
#
# latex.tcl, version 3.2t:  macros, menu, and key bindings for LaTeX users
#
#############################################################################
#
# version 1.1 and 1.2 (11/10/92) by Richard T. Austin <austin@eecs.umich.edu>
# versions 2.0--3.2 and 3.2t (3/97) by Tom Scavo <trscavo@syr.edu>
#
# If you make improvements to this file, please share them!
#
#############################################################################
#############################################################################

#############################################################################
#
# Autoload procedures
#
#############################################################################

proc dummyTeX {} {}
proc latexMenu {} {}

#############################################################################
#
# Flags and variables
#
#############################################################################

# Flags ---
#   syntax:  newModeVar <mode> <name> <initial_value> 1
newModeVar TeX buildPkgsSubmenu 0 1
newModeVar TeX deleteEnvNoisily 1 1
newModeVar TeX deleteObjNoisily 1 1
newModeVar TeX promptNoisily 1 1
newModeVar TeX runTeXInBack 0 1
newModeVar TeX searchNoisily 1 1
newModeVar TeX smartDots 1 1
newModeVar TeX smartQuotes 1 1
newModeVar TeX smartScripts 1 1
# newModeVar TeX useBoxMacro 1 1
newModeVar TeX useBrackets 0 1
newModeVar TeX useDollarSigns 0 1
newModeVar TeX useStatusBar 1 1
newModeVar TeX wordWrap 1 1

# Variables ---
#   syntax:  newModeVar <mode> <name> <initial_value> 0
newModeVar TeX boxMacroNames {includegraphics includegraphics*} 0
newModeVar TeX citeCommands {cite nocite} 0
newModeVar TeX fillColumn {70} 0
# Bug---will not select arguments with embedded braces:
newModeVar TeX funcExpr {\\(sub)*section(\[.*\]|\*)?({[^{}]*})?} 0
newModeVar TeX parseExpr {[^\{]*\{(.*)\}[^\}]*} 0
newModeVar TeX funcExprAlt {\\section(\[.*\]|\*)?({[^{}]*})?} 0 
# newModeVar TeX funcExpr {\\((sub)*section|chapter)(\[.*\]|\*)?({[^{}]*})?} 0
# newModeVar TeX funcExprAlt {\\(section|chapter)(\[.*\]|\*)?({[^{}]*})?} 0 
newModeVar TeX leftFillColumn {0} 0
newModeVar TeX prefixString {% } 0
newModeVar TeX refCommands {ref pageref} 0
# Will not find  \<space> :
# newModeVar TeX wordBreak {([a-zA-Z-0-9]+|\\(\\\*?|[^A-Za-z\t\r* \]|[A-Za-z]+\*?))} 0
# newModeVar TeX wordBreakPreface {([^a-zA-Z-0-9\]|.\\)} 0
# newModeVar TeX wrapBreak {([a-zA-Z-0-9]+|\\(\\\*?|[^A-Za-z\t\r* \]|[A-Za-z]+\*?))} 0
# newModeVar TeX wrapBreakPreface {([^a-zA-Z-0-9\]|.\\)} 0
newModeVar TeX wordBreak {(\w+|\\(\\\*?|[^A-Za-z\t\r*\s\]|[A-Za-z]+\*?))} 0
newModeVar TeX wordBreakPreface {([^\w\]|.\\)} 0
newModeVar TeX wrapBreak {(\w+|\\(\\\*?|[^A-Za-z\t\r*\s\]|[A-Za-z]+\*?))} 0
newModeVar TeX wrapBreakPreface {([^\w\]|.\\)} 0

#############################################################################
#
# Initialization
#
#############################################################################

# Rename and redefine 'changeMode', which is called for all open, 
# saveas, and activate events:
if { [info commands TeXchangeMode] == "" } {
	rename changeMode TeXchangeMode
}
proc changeMode {newMode} {	
	# Do the normal thing:
	TeXchangeMode $newMode
	# Refresh the Typeset submenu:
	catch {eval [typesetSubmenu]}
# 	if { $newMode == "TeX" } {
# # 		dummyTeX
# 		eval [typesetSubmenu]
# 	}
}

# Rename and redefine 'closeHook', which is called for all close events:
if { [info commands TeXcloseHook] == "" } {
	rename closeHook TeXcloseHook
}
proc closeHook {name} {
	global TeXconnections
	if {[info exists TeXconnections]} {
		set winNames [map car $TeXconnections]
		if { [set pos [lsearch -exact $winNames $name]] > -1 } {
			# Close Textures' connection:
			global texAppSig
			set quotedSig "'$texAppSig(Textures)'"
			set TeXjob [cadr [lindex $TeXconnections $pos]]
			AEBuild -r $quotedSig BSRs Disc Jobi $TeXjob
			# Update list of connections:
			set TeXconnections [lreplace $TeXconnections $pos $pos]
		}
	}
	# Do the normal thing:
	TeXcloseHook $name
	# Refresh the Typeset submenu in case the latexMenu 
	# is (or becomes) global:
# 	catch {eval [typesetSubmenu]}
}

# Rename and redefine 'launchBackAppl', which is called by 
# 'launchBackApplSigs' in procs.tcl if the requested application 
# is not running:
if { [info commands TeXlaunchBackAppl] == "" } {
	rename launchBackAppl TeXlaunchBackAppl
}
proc launchBackAppl {sig} {
	global texAppSig
	if {[info exists texAppSig] && $sig == $texAppSig(Textures)} {
		global TeXconnections
		set TeXconnections ""
	}
	# Do the normal thing:
	TeXlaunchBackAppl $sig
}

# The proc 'shadowtexSig' is executed automatically whenever 
# the global variable 'texSig' is altered, say, by choosing
# 'removedSavedSetting' on the Config menu.
trace variable texSig w shadowtexSig
proc shadowtexSig {{name1 {}} {name2 {}} {op w}} {
	global mode latexMenu packagesSubmenuItems TeXSearchPath
	
	# Reset the Packages submenu so that it will be rebuilt:
	set packagesSubmenuItems ""
	
	if { $mode == "TeX" } { removeMenu $latexMenu }
	if { [info commands buildLaTeXMenu] == "" } {
		dummyTeX
	} else {
		message {Rebuilding LaTeX menu}
		buildLaTeXMenu
		message ""
	}
	if { $mode == "TeX" } { insertMenu $latexMenu }
	
	# Reset the search path so it will be rebuilt the next time it's needed:
	set TeXSearchPath {}
	
	# Does this do the right thing?
	lappend modifiedVars texSig
}

# The proc 'shadowTeXInputs' is executed automatically whenever 
# the global variable 'TeXInputs' is altered, say, from the
# Config:appPaths submenu.
trace variable TeXInputs w shadowTeXInputs
proc shadowTeXInputs {{name1 {}} {name2 {}} {op w}} {
# 	global mode latexMenu
	global packagesSubmenuItems TeXSearchPath

	# Reset the Packages submenu so that it will be rebuilt:
	set packagesSubmenuItems ""
	
# 	if { $mode == "TeX" } { removeMenu $latexMenu }
	if { [info commands documentsSubmenu] == "" } {
		dummyTeX
	} else {
		message "Rebuilding the Documents submenu"
		eval [documentsSubmenu]
		message ""
	}
# 	if { $mode == "TeX" } { insertMenu $latexMenu }

	# Reset the search path so it will be rebuilt the next time it's needed:
	set TeXSearchPath {}
}

# The proc 'shadowUseDollarSigns' is executed automatically whenever 
# the flag 'useDollarSigns' is altered.  The proc 'mathModeSubmenu'
# is defined in latexMenu.tcl.
trace variable TeXmodeVars(useDollarSigns) w shadowUseDollarSigns
proc shadowUseDollarSigns {{name1 {}} {name2 {}} {op w}} {
	eval [mathModeSubmenu]
}

#############################################################################
#
# Colorization
#
#############################################################################

newModeVar TeX keywordColor blue 0
newModeVar TeX sectionColor magenta 0
newModeVar TeX commentColor red 0
newModeVar TeX bracesColor green 0

# The proc 'shadowCiteRefCommands' is executed whenever the variables 
# TeXmodeVars(citeCommands) or TeXmodeVars(refCommands) are changed.
trace variable TeXmodeVars(citeCommands) w shadowCiteRefCommands
trace variable TeXmodeVars(refCommands) w shadowCiteRefCommands
proc shadowCiteRefCommands {{name1 {}} {name2 {}} {op w}} {
	global TeXmodeVars
	set LaTeXClickWords {}
	# insure that 'citeCommands' contains "cite" and "nocite":
	if {[lsearch -exact $TeXmodeVars(citeCommands) cite] == -1} then {
		lappend TeXmodeVars(citeCommands) cite
	}
	if {[lsearch -exact $TeXmodeVars(citeCommands) nocite] == -1} then {
		lappend TeXmodeVars(citeCommands) nocite
	}
	foreach word $TeXmodeVars(citeCommands) { 
		append LaTeXClickWords " \\\\$word" 
	}
	# insure that 'refCommands' contains "ref" and "pageref":
	if {[lsearch -exact $TeXmodeVars(refCommands) ref] == -1} then {
		lappend TeXmodeVars(refCommands) ref
	}
	if {[lsearch -exact $TeXmodeVars(refCommands) pageref] == -1} then {
		lappend TeXmodeVars(refCommands) pageref
	}
	foreach word $TeXmodeVars(refCommands) { 
		append LaTeXClickWords " \\\\$word" 
	}
	# color and underline command-double-clickable LaTeX commands:
	regModeKeywords -a -u -k $TeXmodeVars(keywordColor) TeX $LaTeXClickWords
}

# The proc 'shadowBoxMacroNames' is executed whenever the variable 
# TeXmodeVars(boxMacroNames) is changed.
trace variable TeXmodeVars(boxMacroNames) w shadowBoxMacroNames
proc shadowBoxMacroNames {{name1 {}} {name2 {}} {op w}} {
	global TeXmodeVars
	set LaTeXClickWords {}
	foreach word $TeXmodeVars(boxMacroNames) { 
		append LaTeXClickWords " \\\\$word" 
	}
	# color and underline command-double-clickable LaTeX commands:
	regModeKeywords -a -u -k $TeXmodeVars(keywordColor) TeX $LaTeXClickWords
}

proc colorLaTeXClickCommands {} {
	global TeXmodeVars
	set LaTeXClickWords { \\documentclass \\usepackage }
	append LaTeXClickWords { \\input \\include }
	append LaTeXClickWords { \\bibliography \\bibliographystyle }
	append LaTeXClickWords { \\LoadClass \\RequirePackage }
	# color and underline command-double-clickable LaTeX commands:
	regModeKeywords -a -u -k $TeXmodeVars(keywordColor) TeX $LaTeXClickWords
	shadowCiteRefCommands
	shadowBoxMacroNames
}

proc colorLaTeXCommands {} {
	global TeXmodeVars 
	set LaTeXKeyWords {}
	# Color comments, braces and dollar signs, and all LaTeX commands:
	regModeKeywords -i "\}" -i "\{" -i "\$" -I $TeXmodeVars(bracesColor) \
	      -e {%} -m {\\} -c $TeXmodeVars(commentColor) \
	      -k $TeXmodeVars(keywordColor) TeX $LaTeXKeyWords
	# Color sectioning commands:
	regModeKeywords -a -k $TeXmodeVars(sectionColor) TeX \
		{ \\part \\chapter \\section \\subsection \\subsubsection \\paragraph }
}

colorLaTeXCommands; rename colorLaTeXCommands ""
colorLaTeXClickCommands; rename colorLaTeXClickCommands ""

#############################################################################
#
# LaTeX core routines
#
#############################################################################

set isLaTeXLoaded 1
if { [catch {source "$HOME:Tcl:Modes:latexEngine.tcl"}] } {
	set isLaTeXLoaded 0
	beep
	alertnote "Loading of latexEngine.tcl failed!"
}

#############################################################################
#
# LaTeX communications
#
#############################################################################

if { [catch {source "$HOME:Tcl:Modes:latexComm.tcl"}] } {
	set isLaTeXLoaded 0
	beep
	alertnote "Loading of latexComm.tcl failed!"
}

#############################################################################
#
# LaTeX macros
#
#############################################################################

if { [catch {source "$HOME:Tcl:Modes:latexMacros.tcl"}] } {
	set isLaTeXLoaded 0
	beep
	alertnote "Loading of latexMacros.tcl failed!"
}

#############################################################################
#
# LaTeX menu definition
#
#############################################################################

# if { [info commands buildSubMenu] == "" } {
# 	source "$HOME:Tcl:Modes:DblClickAux.tcl"
# }
if { [catch {source "$HOME:Tcl:Modes:latexMenu.tcl"}] } {
	set isLaTeXLoaded 0
	beep
	alertnote "Loading of latexMenu.tcl failed!"
}

#############################################################################
#
# LaTeX key bindings
#
#############################################################################

if { [catch {source "$HOME:Tcl:Modes:latexKeys.tcl"}] } {
	set isLaTeXLoaded 0
	beep
	alertnote "Loading of latexKeys.tcl failed!"
}

#############################################################################
#
# Smart quotes and dots
#
#############################################################################

if { [catch {source "$HOME:Tcl:Modes:latexSmart.tcl"}] } {
	set isLaTeXLoaded 0
	beep
	alertnote "Loading of latexSmart.tcl failed!"
}

#############################################################################
#
# Patches and bug fixes
#
#############################################################################

if { [catch {source "$HOME:Tcl:Modes:latexPatches.tcl"}] } {
	set isLaTeXLoaded 0
	beep
	alertnote "Loading of latexPatches.tcl failed!"
}

#############################################################################
#############################################################################

if { $isLaTeXLoaded } then {
	message {Binding LaTeX keys}
	if { [catch { bindLaTeXKeys }] } {
		set isLaTeXLoaded 0
		beep
		alertnote "Unable to bind the LaTeX keys!"
	} else {
		rename bindLaTeXKeys ""
	}
	message {Building LaTeX menu}
# 	shadowUseDollarSigns
	if { [catch { buildLaTeXMenu }] } {
		set isLaTeXLoaded 0
		beep
		alertnote "Unable to build the LaTeX menu!"
	}
}
if { $isLaTeXLoaded } then {
	message "LaTeX initialization complete"
} else {
	beep
	alertnote "LaTeX initialization failed!"
}
unset isLaTeXLoaded

#===============================================================================
proc parseFuncsTeX {} {
	global funcExpr parseExpr

	set m {}
	set pos 0
	while {[set res [search -s -f 1 -r 1 -i 0 -n $funcExpr $pos]] != ""} {
		set text [getText [car $res] [expr [nextLineStart [cadr $res]] - 1]]
		if {[regexp $parseExpr $text dummy word]} {
			set num [regsub -all sub $text sub dummy]
			lappend m "[format %${num}s {}]$word" [car $res]
		}
		set pos [cadr $res]
	}
	return $m
}

